@@ -306,10 +306,12 @@ sub disconnect ($) {
 
     # Drop database connection at remote end
     my $rdbh = $dbh->{'proxy_dbh'};
-    local $SIG{__DIE__} = 'DEFAULT';
-    local $@;
-    eval { $rdbh->disconnect() };
-    DBD::Proxy::proxy_set_err($dbh, $@) if $@;
+    if ( $rdbh ) {
+        local $SIG{__DIE__} = 'DEFAULT';
+        local $@;
+	eval { $rdbh->disconnect() } ;
+        DBD::Proxy::proxy_set_err($dbh, $@) if $@;
+    }
     
     # Close TCP connect to remote
     # XXX possibly best left till DESTROY? Add a config attribute to choose?
@@ -510,6 +512,7 @@ sub execute ($@) {
 
     # new execute, so delete any cached rows from previous execute
     undef $sth->{'proxy_data'};
+    undef $sth->{'proxy_rows'};
 
     my $rsth = $sth->{proxy_sth};
     my $dbh = $sth->FETCH('Database');
@@ -590,6 +593,8 @@ sub fetch ($) {
 
     my $data = $sth->{'proxy_data'};
 
+    $sth->{'proxy_rows'} = 0 unless defined $sth->{'proxy_rows'};
+
     if(!$data || !@$data) {
 	return undef unless $sth->SUPER::FETCH('Active');
 
@@ -613,13 +618,14 @@ sub fetch ($) {
     my $row = shift @$data;
 
     $sth->SUPER::STORE(Active => 0) if ( $sth->{proxy_cache_only} and !@$data );
+    $sth->{'proxy_rows'}++;
     return $sth->_set_fbav($row);
 }
 *fetchrow_arrayref = \&fetch;
 
 sub rows ($) {
-    my($sth) = @_;
-    $sth->{'proxy_rows'};
+    my $rows = shift->{'proxy_rows'};
+    return (defined $rows) ? $rows : -1;
 }
 
 sub finish ($) {
@@ -735,7 +741,7 @@ DBD::Proxy - A proxy driver for the DBI
 =head1 DESCRIPTION
 
 DBD::Proxy is a Perl module for connecting to a database via a remote
-DBI driver.
+DBI driver. See L<DBD::Gofer> for an alternative with different trade-offs.
 
 This is of course not needed for DBI drivers which already
 support connecting to a remote database, but there are engines which
